--- loncom/publisher/lonupload.pm 2001/11/29 21:51:40 1.8
+++ loncom/publisher/lonupload.pm 2002/09/10 02:31:26 1.14
@@ -1,7 +1,8 @@
+
# The LearningOnline Network with CAPA
# Handler to upload files into construction space
#
-# $Id: lonupload.pm,v 1.8 2001/11/29 21:51:40 matthew Exp $
+# $Id: lonupload.pm,v 1.14 2002/09/10 02:31:26 foxr Exp $
#
# Copyright Michigan State University Board of Trustees
#
@@ -31,9 +32,11 @@
#
# (TeX Content Handler
#
+# YEAR=2000
# 05/29/00,05/30,10/11 Gerd Kortemeyer)
#
# 11/28,11/29,11/30,12/01,12/02,12/04,12/23 Gerd Kortemeyer
+# YEAR=2001
# 03/23 Guy Albertelli
# 03/24,03/29 Gerd Kortemeyer)
#
@@ -41,15 +44,39 @@
#
# 04/05,04/09,05/25,06/23,06/24,08/22 Gerd Kortemeyer
# 11/29 Matthew Hall
+# 12/16 Scott Harrison
+#
+###
package Apache::lonupload;
use strict;
use Apache::File;
use File::Copy;
+use File::Basename;
use Apache::Constants qw(:common :http :methods);
use Apache::loncacc;
+use Apache::loncommon();
+use Apache::Log();
use Apache::lonnet;
+use HTML::Entities();
+
+my $DEBUG=0;
+
+sub Debug {
+
+ # Marshall the parameters.
+
+ my $r = shift;
+ my $log = $r->log;
+ my $message = shift;
+
+ # Put out the indicated message butonly if DEBUG is false.
+
+ if ($DEBUG) {
+ $log->debug($message);
+ }
+}
sub upfile_store {
my $r=shift;
@@ -81,6 +108,10 @@ sub phaseone {
$fn=~s/^\///;
$fn=~s/(\/)+/\//g;
+# Fn is the full path to the destination filename.
+#
+
+ &Debug($r, "Filename for upload: $fn");
if (($fn) && ($fn!~/\/$/)) {
$r->print(
'
');
- # Check for bad extension
+ # Check for bad extension and warn user
if ($fn=~/\.(\w+)$/ &&
- (&Apache::lonnet::fileembstyle($1) eq 'hdn')) {
+ (&Apache::loncommon::fileembstyle($1) eq 'hdn')) {
$r->print(
''.
'The extension on this file, "'.$1.
'", is reserved internally by LON-CAPA.
'.
'Please change the extension.'.
'');
- }
+ } elsif($fn=~/\.(\w+)$/ &&
+ !defined(&Apache::loncommon::fileembstyle($1))) {
+ $r->print(
+ ''.
+ 'The extension on this file, "'.$1.
+ '", is not recognized by LON-CAPA.
'.
+ 'Please change the extension.'.
+ '');
+ }
} else {
$r->print('Illegal filename.');
}
@@ -110,10 +149,20 @@ sub phaseone {
sub phasetwo {
my ($r,$fn,$uname,$udom)=@_;
- if ($fn=~/^\/priv\/$uname\//) {
+ &Debug($r, "Filename is ".$fn);
+ if ($fn=~/^\/priv\/$uname\//) {
+ &Debug($r, "Filename after priv substitution: ".$fn);
my $tfn=$fn;
$tfn=~s/^\/(\~|priv)\/(\w+)//;
+ &Debug($r, "Filename for tfn = ".$tfn);
my $target='/home/'.$uname.'/public_html'.$tfn;
+ &Debug($r, "target -> ".$target);
+# target is the full filesystem path of the destination file.
+ my $base = &File::Basename::basename($fn);
+ my $path = &File::Basename::dirname($fn);
+ $base = &HTML::Entities::encode($base);
+ my $url = $path."/".$base;
+ &Debug($r, "URL is now ".$url);
my $datatoken=$ENV{'form.datatoken'};
if (($fn) && ($datatoken)) {
if ((-e $target) && ($ENV{'form.override'} ne 'Yes')) {
@@ -121,23 +170,31 @@ sub phasetwo {
'');
} else {
my $source=$r->dir_config('lonDaemons').
'/tmp/'.$datatoken.'.tmp';
# Check for bad extension
if ($fn=~/\.(\w+)$/ &&
- (&Apache::lonnet::fileembstyle($1) eq 'hdn')) {
+ (&Apache::loncommon::fileembstyle($1) eq 'hdn')) {
$r->print(
'File '.$fn.' could not be copied.
'.
''.
'The extension on this file is reserved internally by LON-CAPA.'.
'');
+ } elsif ($fn=~/\.(\w+)$/ &&
+ !defined(&Apache::loncommon::fileembstyle($1))) {
+ $r->print(
+ 'File '.$fn.' could not be copied.
'.
+ ''.
+ 'The extension on this file is not recognized by LON-CAPA.'.
+ '');
} elsif (copy($source,$target)) {
+ chmod(0660, $target); # Set permissions to rw-rw---.
$r->print('File copied.');
- $r->print('View file');
} else {
$r->print('Failed to copy: '.$!);
@@ -155,6 +212,7 @@ sub phasetwo {
}
}
+# ---------------------------------------------------------------- Main Handler
sub handler {
my $r=shift;
@@ -216,3 +274,76 @@ sub handler {
1;
__END__
+
+=head1 NAME
+
+Apache::lonupload - upload files into construction space
+
+=head1 SYNOPSIS
+
+Invoked by /etc/httpd/conf/srm.conf:
+
+
+ PerlAccessHandler Apache::lonacc
+ SetHandler perl-script
+ PerlHandler Apache::lonupload
+ ErrorDocument 403 /adm/login
+ ErrorDocument 404 /adm/notfound.html
+ ErrorDocument 406 /adm/unauthorized.html
+ ErrorDocument 500 /adm/errorhandler
+
+
+=head1 INTRODUCTION
+
+This module uploads a file sitting on a client computer into
+library server construction space.
+
+This is part of the LearningOnline Network with CAPA project
+described at http://www.lon-capa.org.
+
+=head1 HANDLER SUBROUTINE
+
+This routine is called by Apache and mod_perl.
+
+=over 4
+
+=item *
+
+Initialize variables
+
+=item *
+
+Start page output
+
+=item *
+
+output relevant interface phase (phaseone or phasetwo)
+
+=item *
+
+(phase one is to specify upload file; phase two is to handle conditions
+subsequent to specification--like overwriting an existing file)
+
+=back
+
+=head1 OTHER SUBROUTINES
+
+=over 4
+
+=item *
+
+phaseone() : Interface for specifying file to upload.
+
+=item *
+
+phasetwo() : Interface for handling post-conditions about uploading (such
+as overwriting an existing file).
+
+=item *
+
+upfile_store() : Store contents of uploaded file into temporary space. Invoked
+by phaseone subroutine.
+
+=back
+
+=cut
@@ -121,23 +170,31 @@ sub phasetwo {
my $source=$r->dir_config('lonDaemons').
'/tmp/'.$datatoken.'.tmp';
- # Check for bad extension
+ # Check for bad extension and disallow upload
if ($fn=~/\.(\w+)$/ &&